public class BankAccountController {
    
    @AuraEnabled
    public static List<BankAccount> getBankAccounts() {
        List<BankAccount> out = new List<BankAccount>();
        
        for (Account__c ba: BankAccountSelector.getAccounts()) {
            out.add(new BankAccount(ba));
        }
        
        return out;
    }
    
    @AuraEnabled
    public static BankAccount getBankAccount(String recordId) {
        return new BankAccount(BankAccountSelector.getAccountWithTransactionsById(recordId));
    }
    
    @AuraEnabled
    public static void saveAccount(String token, Date syncFrom) {
        Try {
            FioConnector.Statement st = FioConnector.readData(token, syncFrom, null);
            
            Account__c acc = st.accountStatement.info.toSobject(); 
            acc.Sync_From__c = Date.valueOf(st.accountStatement.info.dateStart);
            insert acc;
            
            List<Transaction__c> transactions = FioConnector.getTransactions(acc.Id, st);

            upsert transactions External_Id__c;
            
            insert new Setting__c (
                Name = acc.Account_Number__c + '/' + acc.Bank_Code__c,
                Api_Token__c = token
            );
        } catch(Exception e) {
            throw new AuraHandledException(e.getMessage());
        }
    }
    
    @AuraEnabled
    public static void deleteAccount(String recordId) {
        Savepoint sp = Database.setSavepoint();
        
        Try {
            Account__c acc = BankAccountSelector.getAccountById(recordId);
            
            delete BankAccountSelector.getSettingByAccountNumber(acc.Account_Number__c + '/' + acc.Bank_Code__c);
            delete acc;
        } catch(Exception e) {
            Database.rollback(sp);
            throw new AuraHandledException(e.getMessage());
        }
    }
    
    @AuraEnabled
    public static BankAccount syncBalance(String recordId, String accountNumber) {
        Account__c toUpdate = BankAccountSelector.getAccountById(recordId);
        String token = BankAccountSelector.getSettingByAccountNumber(accountNumber).Api_Token__c;
        
        FioConnector.BankAccount a = FioConnector.readData(token, null, null).accountStatement.info;
        
        toUpdate.Balance__c = a.closingBalance;
        toUpdate.Last_Sync__c = System.now();
        toUpdate.Last_Sync_Status__c = Constants.successStatus;
        
        update toUpdate;
        
        return new BankAccount(toUpdate);
    }
    
    @AuraEnabled
    public static BankAccount syncAccount(String recordId) {
        QueueableAccountSync.syncAccount(new List<Account__c>{BankAccountSelector.getAccountById(recordId)});
        
        return new BankAccount(BankAccountSelector.getAccountWithTransactionsById(recordId));
    }
    
    private static List<TransactionW> wrapTransactions(List<Transaction__c> tr) {
        List<TransactionW> out = new List<TransactionW>();
        
        for (Transaction__c t: tr) {
            out.add(new TransactionW(t));
        }
        
        return out;
    }
    
    public class BankAccount {
        @AuraEnabled
        public String recordId;
        @AuraEnabled
        public String name;
        @AuraEnabled
        public Decimal balance;
        @AuraEnabled
        public String accountNumber; 
        @AuraEnabled
        public String bankCode;
        @AuraEnabled
        public DateTime lastSync;
        @AuraEnabled
        public Date syncFrom;
        @AuraEnabled
        public String lastSyncStatus;
        @AuraEnabled
        public String crrncy;
        @AuraEnabled
        public String iban;
        @AuraEnabled
        public List<TransactionW> transactions = new List<TransactionW>();
                
        public BankAccount(Account__c a) {
            this.recordId = a.Id;
            this.name = a.Name__c;
            this.balance = a.Balance__c;
            this.accountNumber = a.Account_Number__c;
            this.bankCode = a.Bank_Code__c;
            this.lastSync = a.Last_Sync__c != null ? a.Last_Sync__c : null;
            this.syncFrom = a.Sync_From__c;
            this.lastSyncStatus = a.Last_Sync_Status__c;
            this.crrncy = a.Currency__c;
            this.iban = a.IBAN__c;
            this.transactions = wrapTransactions(a.Transactions__r);
            transactions.sort();
        }
    }
    
    public class TransactionW implements Comparable {
        @AuraEnabled
        public String recordId;
        @AuraEnabled
        public Date createdDate;
        @AuraEnabled
        public Decimal amount;
        @AuraEnabled
        public String toAccount;
        @AuraEnabled
        public String vs;
        @AuraEnabled
        public String ss;
        @AuraEnabled
        public String ks;
        @AuraEnabled
        public String comment;
        @AuraEnabled
        public String transactionType;
        
        public TransactionW(Transaction__c t) {
            this.recordId = t.Id;
            this.createdDate = t.Date__c;
            this.amount = t.Amount__c;
            this.toAccount = (t.Bank_Account__c == null || t.Bank_Account_Code__c == null) ? '' : t.Bank_Account__c + '/' + t.Bank_Account_Code__c;
            this.vs = t.VS__c;
            this.ss = t.SS__c;
            this.ks = t.KS__c;
            this.comment = t.Comment__c;
            this.transactionType = t.Type__c;
        }
        
        public Integer compareTo(Object compareTo) {
            TransactionW compareToTrans = (TransactionW)compareTo;
            if (createdDate == compareToTrans.createdDate) return 0;
            if (createdDate > compareToTrans.createdDate) return -1;
            return 1;        
        }
    }
}